@@ -1,6 +1,7 @@
 package remotemachineset
 
 import (
+	"bytes"
 	"context"
 	"fmt"
 	"testing"
@@ -13,16 +14,17 @@ import (
 	corev1 "k8s.io/api/core/v1"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/runtime"
+	jsonserializer "k8s.io/apimachinery/pkg/runtime/serializer/json"
 	"k8s.io/apimachinery/pkg/types"
 	"k8s.io/client-go/kubernetes/scheme"
 	"sigs.k8s.io/controller-runtime/pkg/client/fake"
 
 	gcpprovider "github.com/openshift/cluster-api-provider-gcp/pkg/apis/gcpprovider/v1beta1"
-	machineapi "github.com/openshift/cluster-api/pkg/apis/machine/v1beta1"
+	machineapi "github.com/openshift/machine-api-operator/pkg/apis/machine/v1beta1"
 
-	"github.com/openshift/hive/pkg/apis"
-	hivev1 "github.com/openshift/hive/pkg/apis/hive/v1"
-	hivev1gcp "github.com/openshift/hive/pkg/apis/hive/v1/gcp"
+	"github.com/openshift/hive/apis"
+	hivev1 "github.com/openshift/hive/apis/hive/v1"
+	hivev1gcp "github.com/openshift/hive/apis/hive/v1/gcp"
 	"github.com/openshift/hive/pkg/constants"
 	controllerutils "github.com/openshift/hive/pkg/controller/utils"
 	gcpclient "github.com/openshift/hive/pkg/gcpclient"
@@ -32,6 +34,8 @@ import (
 const (
 	gcpCredsSecretName = "gcp-credentials"
 	testProjectID      = "test-gcp-project-id"
+	testNetworkID      = "test-gcp-network-id"
+	testSubnetID       = "test-subnet-id"
 )
 
 func TestGCPActuator(t *testing.T) {
@@ -124,6 +128,49 @@ func TestGCPActuator(t *testing.T) {
 			requireLeases:                   true,
 			setupPendingCreationExpectation: true,
 		},
+		{
+			name: "generate machinesets with explicit disk type and size",
+			pool: func() *hivev1.MachinePool {
+				pool := testGCPPool(testPoolName)
+				pool.Spec.Platform.GCP.OSDisk = hivev1gcp.OSDisk{
+					DiskType:   "faketype",
+					DiskSizeGB: 2000,
+				}
+				return pool
+			}(),
+			mockGCPClient: func(client *mockgcp.MockClient) {
+				mockListComputeZones(client, []string{"zone1"}, testRegion)
+			},
+			expectedMachineSetReplicas: map[string]int64{
+				generateGCPMachineSetName("worker", "zone1"): 3,
+			},
+		},
+		{
+			name: "generate machinesets with KMS disk encryption",
+			pool: func() *hivev1.MachinePool {
+				pool := testGCPPool(testPoolName)
+				pool.Spec.Platform.GCP.OSDisk = hivev1gcp.OSDisk{
+					DiskType:   "faketype",
+					DiskSizeGB: 2000,
+					EncryptionKey: &hivev1gcp.EncryptionKeyReference{
+						KMSKey: &hivev1gcp.KMSKeyReference{
+							Name:      "foo",
+							KeyRing:   "keyring",
+							ProjectID: "myproject",
+							Location:  "Canada",
+						},
+						KMSKeyServiceAccount: "kmssa",
+					},
+				}
+				return pool
+			}(),
+			mockGCPClient: func(client *mockgcp.MockClient) {
+				mockListComputeZones(client, []string{"zone1"}, testRegion)
+			},
+			expectedMachineSetReplicas: map[string]int64{
+				generateGCPMachineSetName("worker", "zone1"): 3,
+			},
+		},
 	}
 
 	for _, test := range tests {
@@ -161,6 +208,8 @@ func TestGCPActuator(t *testing.T) {
 				expectations:   controllerExpectations,
 				projectID:      testProjectID,
 				leasesRequired: test.requireLeases,
+				network:        testNetworkID,
+				subnet:         testSubnetID,
 			}
 
 			generatedMachineSets, _, err := ga.GenerateMachineSets(clusterDeployment, test.pool, ga.logger)
@@ -168,7 +217,46 @@ func TestGCPActuator(t *testing.T) {
 			if test.expectedErr {
 				assert.Error(t, err, "expected error for test case")
 			} else {
-				validateGCPMachineSets(t, generatedMachineSets, test.expectedMachineSetReplicas)
+				assert.Equal(t, len(test.expectedMachineSetReplicas), len(generatedMachineSets), "different number of machine sets generated than expected")
+
+				for _, ms := range generatedMachineSets {
+					expectedReplicas, ok := test.expectedMachineSetReplicas[ms.Name]
+					if assert.True(t, ok, "unexpected machine set: ", ms.Name) {
+						assert.Equal(t, expectedReplicas, int64(*ms.Spec.Replicas), "replica mismatch")
+					}
+
+					gcpProvider, ok := ms.Spec.Template.Spec.ProviderSpec.Value.Object.(*gcpprovider.GCPMachineProviderSpec)
+					assert.True(t, ok, "failed to convert to gcpProviderSpec")
+
+					assert.Equal(t, testInstanceType, gcpProvider.MachineType, "unexpected instance type")
+
+					// Ensure network details are propagated correctly.
+					assert.Equal(t, ga.network, gcpProvider.NetworkInterfaces[0].Network)
+					assert.Equal(t, ga.subnet, gcpProvider.NetworkInterfaces[0].Subnetwork)
+
+					// Ensure GCP disk type and size was correctly set or defaulted and made it to the resulting MachineSets:
+					expectedDiskType := test.pool.Spec.Platform.GCP.OSDisk.DiskType
+					expectedDiskSizeGB := test.pool.Spec.Platform.GCP.OSDisk.DiskSizeGB
+					if expectedDiskType == "" {
+						expectedDiskType = defaultGCPDiskType
+					}
+					if expectedDiskSizeGB == 0 {
+						expectedDiskSizeGB = defaultGCPDiskSizeGB
+					}
+					assert.Equal(t, expectedDiskType, gcpProvider.Disks[0].Type)
+					assert.Equal(t, expectedDiskSizeGB, gcpProvider.Disks[0].SizeGb)
+
+					// Ensure GCP disk encryption settings made it to the resulting MachineSet (if specified):
+					encKey := test.pool.Spec.Platform.GCP.OSDisk.EncryptionKey
+					if encKey != nil {
+						assert.Equal(t, encKey.KMSKeyServiceAccount, gcpProvider.Disks[0].EncryptionKey.KMSKeyServiceAccount)
+						assert.Equal(t, encKey.KMSKey.Name, gcpProvider.Disks[0].EncryptionKey.KMSKey.Name)
+						assert.Equal(t, encKey.KMSKey.KeyRing, gcpProvider.Disks[0].EncryptionKey.KMSKey.KeyRing)
+						assert.Equal(t, encKey.KMSKey.ProjectID, gcpProvider.Disks[0].EncryptionKey.KMSKey.ProjectID)
+						assert.Equal(t, encKey.KMSKey.Location, gcpProvider.Disks[0].EncryptionKey.KMSKey.Location)
+					}
+
+				}
 			}
 		})
 	}
@@ -497,13 +585,10 @@ func TestObtainLeaseChar(t *testing.T) {
 			require.Contains(t, test.expectedCharIn, string(leaseChar))
 
 			if test.expectCondition != nil {
-				for _, cond := range pool.Status.Conditions {
-					assert.Equal(t, cond.Type, test.expectCondition.Type)
-					assert.Equal(t, cond.Status, test.expectCondition.Status)
+				cond := controllerutils.FindMachinePoolCondition(pool.Status.Conditions, test.expectCondition.Type)
+				if assert.NotNilf(t, cond, "did not find expected condition type: %v", test.expectCondition.Type) {
+					assert.Equal(t, test.expectCondition.Status, cond.Status, "condition found with unexpected status")
 				}
-			} else {
-				// Assuming if you didn't expect a condition, there shouldn't be any.
-				assert.Equal(t, 0, len(pool.Status.Conditions))
 			}
 
 			if test.expectProceed {
@@ -598,20 +683,130 @@ func TestRequireLeases(t *testing.T) {
 	}
 }
 
-func validateGCPMachineSets(t *testing.T, mSets []*machineapi.MachineSet, expectedMSReplicas map[string]int64) {
-	assert.Equal(t, len(expectedMSReplicas), len(mSets), "different number of machine sets generated than expected")
-
-	for _, ms := range mSets {
-		expectedReplicas, ok := expectedMSReplicas[ms.Name]
-		if assert.True(t, ok, "unexpected machine set: ", ms.Name) {
-			assert.Equal(t, expectedReplicas, int64(*ms.Spec.Replicas), "replica mismatch")
-		}
-
-		gcpProvider, ok := ms.Spec.Template.Spec.ProviderSpec.Value.Object.(*gcpprovider.GCPMachineProviderSpec)
-		assert.True(t, ok, "failed to convert to gcpProviderSpec")
-
-		assert.Equal(t, testInstanceType, gcpProvider.MachineType, "unexpected instance type")
+func TestGetNetwork(t *testing.T) {
+	cases := []struct {
+		name              string
+		remoteMachineSets []machineapi.MachineSet
+		expectError       bool
+	}{
+		{
+			name:              "valid remote machinesets",
+			remoteMachineSets: []machineapi.MachineSet{mockMachineSet("worker1", "worker", false, 1, 0)},
+		},
+		{
+			name: "invalid remote machineset",
+			remoteMachineSets: func() []machineapi.MachineSet {
+				ms := []machineapi.MachineSet{mockMachineSet("worker1", "worker", false, 1, 0)}
+				ms[0].Spec.Template.Spec.ProviderSpec.Value = nil
+				return ms
+			}(),
+			expectError: true,
+		},
 	}
+	for _, tc := range cases {
+		t.Run(tc.name, func(t *testing.T) {
+			scheme := runtime.NewScheme()
+			machineapi.SchemeBuilder.AddToScheme(scheme)
+			gcpprovider.SchemeBuilder.AddToScheme(scheme)
+			network, subnet, actualErr := getNetwork(tc.remoteMachineSets, scheme, log.StandardLogger())
+			if tc.expectError {
+				assert.Error(t, actualErr, "expected an error")
+			} else {
+				if assert.NoError(t, actualErr, "unexpected error") {
+					assert.Equal(t, testNetworkID, network, "unexpected network ID")
+					assert.Equal(t, testSubnetID, subnet, "unexpected subnet ID")
+				}
+			}
+		})
+	}
+}
+
+func mockMachineSet(name string, machineType string, unstompedAnnotation bool, replicas int, generation int) machineapi.MachineSet {
+	msReplicas := int32(replicas)
+	ms := machineapi.MachineSet{
+		TypeMeta: metav1.TypeMeta{
+			APIVersion: machineapi.SchemeGroupVersion.String(),
+			Kind:       "MachineSet",
+		},
+		ObjectMeta: metav1.ObjectMeta{
+			Name:      name,
+			Namespace: machineAPINamespace,
+			Labels: map[string]string{
+				machinePoolNameLabel:                       machineType,
+				"machine.openshift.io/cluster-api-cluster": testInfraID,
+				constants.HiveManagedLabel:                 "true",
+			},
+			Generation: int64(generation),
+		},
+
+		Spec: machineapi.MachineSetSpec{
+			Replicas: &msReplicas,
+			Template: machineapi.MachineTemplateSpec{
+				Spec: mockMachineSpec(machineType),
+			},
+		},
+	}
+	// Add a pre-existing annotation which we will ensure remains in updated machinesets.
+	if unstompedAnnotation {
+		ms.Annotations = map[string]string{
+			"hive.openshift.io/unstomped": "true",
+		}
+	}
+	return ms
+}
+
+func mockMachineSpec(machineType string) machineapi.MachineSpec {
+	rawGCPProviderSpec, err := encodeGCPMachineProviderSpec(testGCPProviderSpec(), scheme.Scheme)
+	if err != nil {
+		log.WithError(err).Fatal("error encoding GCP machine provider spec")
+	}
+	return machineapi.MachineSpec{
+		ObjectMeta: machineapi.ObjectMeta{
+			Labels: map[string]string{
+				"machine.openshift.io/cluster-api-cluster":      testInfraID,
+				"machine.openshift.io/cluster-api-machine-role": machineType,
+				"machine.openshift.io/cluster-api-machine-type": machineType,
+			},
+		},
+		ProviderSpec: machineapi.ProviderSpec{
+			Value: rawGCPProviderSpec,
+		},
+
+		Taints: []corev1.Taint{
+			{
+				Key:    "foo",
+				Value:  "bar",
+				Effect: corev1.TaintEffectNoSchedule,
+			},
+		},
+	}
+}
+
+func testGCPProviderSpec() *gcpprovider.GCPMachineProviderSpec {
+	return &gcpprovider.GCPMachineProviderSpec{
+		TypeMeta: metav1.TypeMeta{
+			Kind:       "GCPMachineProviderSpec",
+			APIVersion: gcpprovider.SchemeGroupVersion.String(),
+		},
+		NetworkInterfaces: []*gcpprovider.GCPNetworkInterface{
+			{
+				Network:    testNetworkID,
+				Subnetwork: testSubnetID,
+			},
+		},
+	}
+}
+
+func encodeGCPMachineProviderSpec(gcoProviderSpec *gcpprovider.GCPMachineProviderSpec, scheme *runtime.Scheme) (*runtime.RawExtension, error) {
+	serializer := jsonserializer.NewSerializer(jsonserializer.DefaultMetaFactory, scheme, scheme, false)
+	var buffer bytes.Buffer
+	err := serializer.Encode(gcoProviderSpec, &buffer)
+	if err != nil {
+		return nil, err
+	}
+	return &runtime.RawExtension{
+		Raw: buffer.Bytes(),
+	}, nil
 }
 
 func mockListComputeZones(gClient *mockgcp.MockClient, zones []string, region string) {
