@@ -13,6 +13,7 @@ import (
 	"github.com/golang/mock/gomock"
 	log "github.com/sirupsen/logrus"
 	"github.com/stretchr/testify/assert"
+	"github.com/stretchr/testify/require"
 
 	corev1 "k8s.io/api/core/v1"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
@@ -26,13 +27,13 @@ import (
 	"sigs.k8s.io/controller-runtime/pkg/client/fake"
 	"sigs.k8s.io/controller-runtime/pkg/reconcile"
 
-	machineapi "github.com/openshift/cluster-api/pkg/apis/machine/v1beta1"
 	autoscalingv1 "github.com/openshift/cluster-autoscaler-operator/pkg/apis/autoscaling/v1"
 	autoscalingv1beta1 "github.com/openshift/cluster-autoscaler-operator/pkg/apis/autoscaling/v1beta1"
+	machineapi "github.com/openshift/machine-api-operator/pkg/apis/machine/v1beta1"
 
-	"github.com/openshift/hive/pkg/apis"
-	hivev1 "github.com/openshift/hive/pkg/apis/hive/v1"
-	hivev1aws "github.com/openshift/hive/pkg/apis/hive/v1/aws"
+	"github.com/openshift/hive/apis"
+	hivev1 "github.com/openshift/hive/apis/hive/v1"
+	hivev1aws "github.com/openshift/hive/apis/hive/v1/aws"
 	"github.com/openshift/hive/pkg/constants"
 	"github.com/openshift/hive/pkg/controller/remotemachineset/mock"
 	"github.com/openshift/hive/pkg/remoteclient"
@@ -616,6 +617,33 @@ func TestRemoteMachineSetReconcile(t *testing.T) {
 				*testClusterAutoscaler("1"),
 			},
 		},
+		{
+			name:              "Create machine autoscalers with zero minReplicas",
+			clusterDeployment: testClusterDeployment(),
+			machinePool:       testAutoscalingMachinePool(0, 5),
+			remoteExisting: []runtime.Object{
+				testMachine("master1", "master"),
+				testClusterAutoscaler("1"),
+			},
+			generatedMachineSets: []*machineapi.MachineSet{
+				testMachineSet("foo-12345-worker-us-east-1a", "worker", false, 0, 0),
+				testMachineSet("foo-12345-worker-us-east-1b", "worker", false, 0, 0),
+				testMachineSet("foo-12345-worker-us-east-1c", "worker", false, 0, 0),
+			},
+			expectedRemoteMachineSets: []*machineapi.MachineSet{
+				testMachineSet("foo-12345-worker-us-east-1a", "worker", false, 0, 0),
+				testMachineSet("foo-12345-worker-us-east-1b", "worker", false, 0, 0),
+				testMachineSet("foo-12345-worker-us-east-1c", "worker", false, 0, 0),
+			},
+			expectedRemoteMachineAutoscalers: []autoscalingv1beta1.MachineAutoscaler{
+				*testMachineAutoscaler("foo-12345-worker-us-east-1a", "1", 0, 2),
+				*testMachineAutoscaler("foo-12345-worker-us-east-1b", "1", 0, 2),
+				*testMachineAutoscaler("foo-12345-worker-us-east-1c", "1", 0, 1),
+			},
+			expectedRemoteClusterAutoscalers: []autoscalingv1.ClusterAutoscaler{
+				*testClusterAutoscaler("1"),
+			},
+		},
 	}
 
 	for _, test := range tests {
@@ -631,8 +659,8 @@ func TestRemoteMachineSetReconcile(t *testing.T) {
 			if test.machinePool != nil {
 				localExisting = append(localExisting, test.machinePool)
 			}
-			fakeClient := fake.NewFakeClient(localExisting...)
-			remoteFakeClient := fake.NewFakeClient(test.remoteExisting...)
+			fakeClient := fake.NewClientBuilder().WithRuntimeObjects(localExisting...).Build()
+			remoteFakeClient := fake.NewClientBuilder().WithRuntimeObjects(test.remoteExisting...).Build()
 
 			mockCtrl := gomock.NewController(t)
 			defer mockCtrl.Finish()
@@ -660,7 +688,7 @@ func TestRemoteMachineSetReconcile(t *testing.T) {
 				},
 				expectations: controllerExpectations,
 			}
-			_, err := rcd.Reconcile(reconcile.Request{
+			_, err := rcd.Reconcile(context.TODO(), reconcile.Request{
 				NamespacedName: types.NamespacedName{
 					Name:      fmt.Sprintf("%s-worker", testName),
 					Namespace: testNamespace,
@@ -750,6 +778,84 @@ func TestRemoteMachineSetReconcile(t *testing.T) {
 	}
 }
 
+func Test_summarizeMachinesError(t *testing.T) {
+	cases := []struct {
+		name     string
+		existing []runtime.Object
+
+		reason, message string
+	}{{
+		name: "no matching machines",
+		existing: []runtime.Object{
+			testMachine("machine-1", "worker"),
+			testMachine("machine-2", "worker"),
+			testMachine("machine-3", "worker"),
+			testMachineSet(testName, "worker", false, 3, 0),
+		},
+	}, {
+		name: "all matching machines ok",
+		existing: []runtime.Object{
+			testMachineSetMachine("machine-1", "worker", testName),
+			testMachineSetMachine("machine-2", "worker", testName),
+			testMachineSetMachine("machine-3", "worker", testName),
+			testMachineSet(testName, "worker", false, 3, 0),
+		},
+	}, {
+		name: "one machine failed",
+		existing: []runtime.Object{
+			testMachineSetMachine("machine-1", "worker", testName),
+			func() *machineapi.Machine {
+				m := testMachineSetMachine("machine-2", "worker", testName)
+				m.Status.ErrorReason = (*machineapi.MachineStatusError)(pointer.StringPtr("GoneNotComingBack"))
+				m.Status.ErrorMessage = pointer.StringPtr("The machine is not found")
+				return m
+			}(),
+			testMachineSetMachine("machine-3", "worker", testName),
+			testMachineSet(testName, "worker", false, 3, 0),
+		},
+
+		reason:  "GoneNotComingBack",
+		message: "The machine is not found",
+	}, {
+		name: "more than one machine failed",
+		existing: []runtime.Object{
+			testMachineSetMachine("machine-1", "worker", testName),
+			func() *machineapi.Machine {
+				m := testMachineSetMachine("machine-2", "worker", testName)
+				m.Status.ErrorReason = (*machineapi.MachineStatusError)(pointer.StringPtr("GoneNotComingBack"))
+				m.Status.ErrorMessage = pointer.StringPtr("The machine is not found")
+				return m
+			}(),
+			func() *machineapi.Machine {
+				m := testMachineSetMachine("machine-3", "worker", testName)
+				m.Status.ErrorReason = (*machineapi.MachineStatusError)(pointer.StringPtr("InsufficientResources"))
+				m.Status.ErrorMessage = pointer.StringPtr("No available quota")
+				return m
+			}(),
+			testMachineSet(testName, "worker", false, 3, 0),
+		},
+		reason: "MultipleMachinesFailed",
+		message: `Machine machine-2 failed (GoneNotComingBack): The machine is not found,
+Machine machine-3 failed (InsufficientResources): No available quota,
+`,
+	}}
+
+	machineapi.SchemeBuilder.AddToScheme(scheme.Scheme)
+	for _, test := range cases {
+		t.Run(test.name, func(t *testing.T) {
+			fake := fake.NewClientBuilder().WithRuntimeObjects(test.existing...).Build()
+
+			ms := &machineapi.MachineSet{}
+			err := fake.Get(context.TODO(), types.NamespacedName{Namespace: machineAPINamespace, Name: testName}, ms)
+			require.NoError(t, err)
+
+			reason, message := summarizeMachinesError(fake, ms, log.WithField("controller", "remotemachineset"))
+			assert.Equal(t, test.reason, reason)
+			assert.Equal(t, test.message, message)
+		})
+	}
+}
+
 func testMachinePool() *hivev1.MachinePool {
 	return &hivev1.MachinePool{
 		TypeMeta: metav1.TypeMeta{
@@ -785,6 +891,26 @@ func testMachinePool() *hivev1.MachinePool {
 				},
 			},
 		},
+		Status: hivev1.MachinePoolStatus{
+			Conditions: []hivev1.MachinePoolCondition{
+				{
+					Status: corev1.ConditionUnknown,
+					Type:   hivev1.NotEnoughReplicasMachinePoolCondition,
+				},
+				{
+					Status: corev1.ConditionUnknown,
+					Type:   hivev1.NoMachinePoolNameLeasesAvailable,
+				},
+				{
+					Status: corev1.ConditionUnknown,
+					Type:   hivev1.InvalidSubnetsMachinePoolCondition,
+				},
+				{
+					Status: corev1.ConditionUnknown,
+					Type:   hivev1.UnsupportedConfigurationMachinePoolCondition,
+				},
+			},
+		},
 	}
 }
 
@@ -795,6 +921,14 @@ func testAutoscalingMachinePool(min, max int) *hivev1.MachinePool {
 		MinReplicas: int32(min),
 		MaxReplicas: int32(max),
 	}
+	for i, cond := range p.Status.Conditions {
+		// Condition will always be present because it is initialized in testMachinePool
+		if cond.Type == hivev1.NotEnoughReplicasMachinePoolCondition {
+			cond.Status = corev1.ConditionFalse
+			cond.Reason = "EnoughReplicas"
+			p.Status.Conditions[i] = cond
+		}
+	}
 	return p
 }
 
@@ -816,7 +950,7 @@ func testMachineSpec(machineType string) machineapi.MachineSpec {
 		log.WithError(err).Fatal("error encoding AWS machine provider spec")
 	}
 	return machineapi.MachineSpec{
-		ObjectMeta: metav1.ObjectMeta{
+		ObjectMeta: machineapi.ObjectMeta{
 			Labels: map[string]string{
 				"machine.openshift.io/cluster-api-cluster":      testInfraID,
 				"machine.openshift.io/cluster-api-machine-role": machineType,
@@ -840,12 +974,22 @@ func testMachineSpec(machineType string) machineapi.MachineSpec {
 func testMachine(name string, machineType string) *machineapi.Machine {
 	return &machineapi.Machine{
 		ObjectMeta: metav1.ObjectMeta{
-			Name: name,
+			Name:      name,
+			Namespace: machineAPINamespace,
+			Labels: map[string]string{
+				"machine.openshift.io/cluster-api-cluster": testInfraID,
+			},
 		},
 		Spec: testMachineSpec(machineType),
 	}
 }
 
+func testMachineSetMachine(name string, machineType string, machineSetName string) *machineapi.Machine {
+	m := testMachine(name, machineType)
+	m.ObjectMeta.Labels["machine.openshift.io/cluster-api-machineset"] = machineSetName
+	return m
+}
+
 func testMachineSet(name string, machineType string, unstompedAnnotation bool, replicas int, generation int) *machineapi.MachineSet {
 	msReplicas := int32(replicas)
 	ms := machineapi.MachineSet{
@@ -865,7 +1009,19 @@ func testMachineSet(name string, machineType string, unstompedAnnotation bool, r
 		},
 		Spec: machineapi.MachineSetSpec{
 			Replicas: &msReplicas,
+			Selector: metav1.LabelSelector{
+				MatchLabels: map[string]string{
+					"machine.openshift.io/cluster-api-machineset": name,
+					"machine.openshift.io/cluster-api-cluster":    testInfraID,
+				},
+			},
 			Template: machineapi.MachineTemplateSpec{
+				ObjectMeta: machineapi.ObjectMeta{
+					Labels: map[string]string{
+						"machine.openshift.io/cluster-api-machineset": name,
+						"machine.openshift.io/cluster-api-cluster":    testInfraID,
+					},
+				},
 				Spec: testMachineSpec(machineType),
 			},
 		},
